
from requests_oauthlib import OAuth2Session
from . import exceptions
import json
import requests

class OuraOAuth2Client:
    """
    Use this for authorizing user and obtaining initial access and refresh token.
    Should be one time usage per user.
    """

    AUTHORIZE_BASE_URL = "https://cloud.ouraring.com/oauth/authorize"
    TOKEN_BASE_URL = "https://api.ouraring.com/oauth/token"
    SCOPE = ["email", "personal", "daily"]

    def __init__(self, client_id, client_secret):

        """
        Initialize the client for oauth flow.
        
        :param client_id: The client id from oura portal.
        :type client_id: str
        :param client_secret: The client secret from oura portal.
        :type client_secret: str
        """
        self.client_id = client_id
        self.client_secret = client_secret

        self.session = OAuth2Session(
            client_id,
            auto_refresh_url=self.TOKEN_BASE_URL,
        )


    def authorize_endpoint(self, scope=None, redirect_uri=None, **kwargs):
        """
        Build the authorization url for a user to click.

        :param scope: Scopes to request from the user. Defaults to self.SCOPE
        :type scope: str
        :param redirect_uri: Where to redirect after user grants access.
        :type redirect_uri: str
        """
        self.session.scope = scope or self.SCOPE
        if redirect_uri:
            self.session.redirect_uri = redirect_uri
        return self.session.authorization_url(self.AUTHORIZE_BASE_URL, **kwargs)


    def fetch_access_token(self, code):
        """
        Exchange the auth code for an access and refresh token. 

        :param code: Authorization code from query string
        :type code: str
        """
        return self.session.fetch_token(
            self.TOKEN_BASE_URL,
            code=code,
            client_secret = self.client_secret)


class OuraClient:
    """
    Use this class for making requests on behalf of a user. If refresh_token and expires_at are supplied, 
    access_token should be refreshed automatically and passed to the refresh_callback function, along with
    other properties in the response.
    """

    API_ENDPOINT = "https://api.ouraring.com"
    TOKEN_BASE_URL = "https://api.ouraring.com/oauth/token"

    def __init__(self, client_id, access_token=None, refresh_token=None,
        expires_at=None, refresh_callback=None):

        """
        Initialize the client

        :param client_id: The client id from oura portal.
        :type client_id: str

        :param access_token: Auth token.
        :type access_token: str

        :param refresh_token: Use this to renew tokens when they expire
        :type refresh_token: str

        :param expires_at: The unix timestamp for access token expiration.
        :type expires_at: str

        :param refresh_callback: Method to save the access token, refresh token, expires at
        :type refresh_callback: callable

        """

        token = {}
        if access_token:
            token.update({ 'access_token': access_token })
        if refresh_token:
            token.update({ 'refresh_token': refresh_token })
        if expires_at:
            token['expires_at'] = expires_at

        self._session = OAuth2Session(
            client_id,
            token=token,
            auto_refresh_url=self.TOKEN_BASE_URL,
            token_updater=refresh_callback
        )


    def user_info(self):
        """
        Returns information about the logged in user (who the access token was issued for).

        See https://cloud.ouraring.com/docs/personal-info
        """
        url = "{}/v1/userinfo".format(self.API_ENDPOINT)
        return self._make_request(url)

    
    def sleep_summary(self, start=None, end=None):
        """
        Get sleep summary for the given date range. See https://cloud.ouraring.com/docs/sleep

        :param start: Beginning of date range
        :type start: date

        :param end: End of date range, or None if you want the current day.
        :type end: date
        """
        url = self._build_summary_url(start, end, "sleep")
        return self._make_request(url)


    def activity_summary(self, start=None, end=None):
        """
        Get activity summary for the given date range. See https://cloud.ouraring.com/docs/activity

        :param start: Beginning of date range
        :type start: date

        :param end: End of date range, or None if you want the current day.
        :type end: date
        """
        url = self._build_summary_url(start, end, "activity")
        return self._make_request(url)


    def readiness_summary(self, start=None, end=None):
        """
        Get readiness summary for the given date range. See https://cloud.ouraring.com/docs/readiness

        :param start: Beginning of date range
        :type start: date

        :param end: End of date range, or None if you want the current day.
        :type end: date
        """
        url = self._build_summary_url(start, end, "readiness")
        return self._make_request(url)


    def _make_request(self, url, data=None, method=None, **kwargs):
        data = data or {}
        method = method or 'GET'
        response = self._session.request(
            method,
            url,
            data=data,
            **kwargs
        )
        exceptions.detect_and_raise_error(response)
        payload = json.loads(response.content.decode('utf8'))
        return payload

    def _build_summary_url(self, start, end, datatype):
        if start is None:
            raise ValueError("Request for {} summary must include start date.".format(datatype))

        url = "{0}/v1/{1}?start={2}".format(self.API_ENDPOINT, datatype, start)
        if end:
            url = "{0}&end={1}".format(url, end)
        return url


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must:
#   $ pip install twine

import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = 'oura'
DESCRIPTION = 'Oura api client.'
URL = 'https://github.com/turing-complet/python-ouraring'
EMAIL = 'jhagg314@gmail.com'
AUTHOR = 'Jon Hagg'
REQUIRES_PYTHON = '>=3.5.3'
VERSION = '1.0.2'

REQUIRED = [
    'requests-oauthlib'
]

EXTRAS = {
    # 'fancy feature': ['django'],
}

here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

about = {}
about['__version__'] = VERSION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = [
        ('test', None, 'Upload to test server')
    ]

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        self.test = False
        self.test_server = 'https://test.pypi.org/legacy/'

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        if self.test:
            self.status('Uploading the package to test server via Twine…')
            os.system('twine upload --repository-url {} dist/*'.format(self.test_server))
        else:
            self.status('Uploading the package to PyPI via Twine…')
            os.system('twine upload dist/*')

        # self.status('Pushing git tags…')
        # os.system('git tag v{0}'.format(about['__version__']))
        # os.system('git push --tags')
        
        sys.exit()


# Where the magic happens:
setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=('tests',)),
    # If your package is a single module, use this instead of 'packages':
    # py_modules=['oura.client'],

    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license='MIT',
    # $ setup.py publish support.
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    cmdclass={
        'upload': UploadCommand,
    },
)


import json

class Timeout(Exception):
    """
    Used when a timeout occurs.
    """
    pass


class HTTPException(Exception):
    def __init__(self, response, *args, **kwargs):
        try:
            errors = json.loads(response.content.decode('utf8'))['errors']
            message = '\n'.join([error['message'] for error in errors])
        except Exception:
            if hasattr(response, 'status_code') and response.status_code == 401:
                message = response.content.decode('utf8')
            else:
                message = response
        super(HTTPException, self).__init__(message, *args, **kwargs)


class HTTPBadRequest(HTTPException):
    """Generic >= 400 error
    """
    pass


class HTTPUnauthorized(HTTPException):
    """401
    """
    pass


class HTTPForbidden(HTTPException):
    """403
    """
    pass


class HTTPNotFound(HTTPException):
    """404
    """
    pass


class HTTPConflict(HTTPException):
    """409 - returned when creating conflicting resources
    """
    pass


class HTTPTooManyRequests(HTTPException):
    """429 - returned when exceeding rate limits
    """
    pass


class HTTPServerError(HTTPException):
    """Generic >= 500 error
    """
    pass


def detect_and_raise_error(response):
    if response.status_code == 401:
        raise HTTPUnauthorized(response)
    elif response.status_code == 403:
        raise HTTPForbidden(response)
    elif response.status_code == 404:
        raise HTTPNotFound(response)
    elif response.status_code == 409:
        raise HTTPConflict(response)
    elif response.status_code == 429:
        exc = HTTPTooManyRequests(response)
        exc.retry_after_secs = int(response.headers['Retry-After'])
        raise exc
    elif response.status_code >= 500:
        raise HTTPServerError(response)
    elif response.status_code >= 400:
        raise HTTPBadRequest(response)
